#!/bin/sh
script_name=check_card_bsp
card_type=SXC4
script_ver=v1.2
###############################################################################################
#version 1.0 support SXC4 BSP 2017112301
#Version 1.1 a) Add RCW 600M to adapt 1G DDR
#            b) update rootfs to delete vender's name
#            c) update tdb for rtc issue
#
#version 1.2 improvemnt to get /dev/mtd0 rcw md5
#
#
###############################################################################################

#expected information for SXC4 cardtype 102
VERIFY_DTB_VER="dtb_sxc4_common-2017120801"
VERIFY_ROOTFS_VER="20171207.102614"
VERIFY_KERNEL_VER="kernel_t1024-3.12.37-2017112201"
VERIFY_UBOOT_common_VER="uboot_sxc4_common-2017112201"
VERIFY_UBOOT_1G_VER="uboot_sxc4_1g-2017112201"
VERIFY_RCW_MD5SUM_600M="cac33c192ed5f163804ed95d5c90c7f3"
VERIFY_RCW_MD5SUM_DEF="b16ccc99d5f3002a921ef0b03fadb1cd"

# get Current Cardtype and info
INFO_MEM_SIZE=$(cat /proc/meminfo | grep MemTotal | grep -o "[0-9.]*")
INFO_DTB_VER=$(cat /proc/device-tree/version)
INFO_ROOTFS_VER=$(cat /etc/st_rootfs_spec | grep buildtime | grep -o "[0-9.]*")
INFO_KERNEL_VER=$(uname)
INFO_UBOOT_VER=$(fw_printenv | grep ^version= | cut -d= -f2)

dd if=/dev/mtd0 of=/tmp/rcw.img ibs=80 count=1
INFO_RCW_MD5SUM=$(md5sum /tmp/rcw.img | cut -d ' ' -f 1)
rm /tmp/rcw.img

if [ $INFO_MEM_SIZE -gt 500000 ]; then
VERIFY_UBOOT_VER=$VERIFY_UBOOT_1G_VER
VERIFY_RCW_MD5SUM=$VERIFY_RCW_MD5SUM_600M
Mem_size="1G"
echo 
else
VERIFY_UBOOT_VER=$VERIFY_UBOOT_common_VER
VERIFY_RCW_MD5SUM=$VERIFY_RCW_MD5SUM_DEF
Mem_size="512M"
fi

echo ""
echo "sctipts $script_name for $card_type version $script_ver"
echo "    current card info:"
echo "    MEM   size: $INFO_MEM_SIZE kB      $Mem_size"


if [ "$INFO_DTB_VER" = $VERIFY_DTB_VER ]; then
   Result_info="Pass.."
else
   Result_info="Miss match!!! Expect version $VERIFY_DTB_VER"
fi
echo "    DTB    ver: $INFO_DTB_VER $Result_info"

if [ "$INFO_ROOTFS_VER" = $VERIFY_ROOTFS_VER ]; then
Result_info="Pass.."
else
Result_info="Miss match!!! Expect version $VERIFY_ROOTFS_VER"
fi
echo "    ROOTFS ver: $INFO_ROOTFS_VER  $Result_info"

if [ "$INFO_KERNEL_VER" = $VERIFY_KERNEL_VER ]; then
Result_info="Pass.."
else
Result_info="Miss match!!! Expect version $VERIFY_KERNEL_VER"
fi

echo "    KERNEL ver: $INFO_KERNEL_VER  $Result_info"

if [ "$INFO_UBOOT_VER" = $VERIFY_UBOOT_VER ]; then
Result_info="Pass.."
else
Result_info="Miss match!!! Expect version $VERIFY_UBOOT_VER"
fi
echo "    UBOOT  ver: $INFO_UBOOT_VER   $Result_info"

if [ "$INFO_RCW_MD5SUM" = $VERIFY_RCW_MD5SUM ]; then
Result_info="Pass.."
else
Result_info="Miss match!!! Expect version $VERIFY_RCW_MD5SUM"
fi
echo "    RCW md5sum: $INFO_RCW_MD5SUM  $Result_info"
echo ""

