#!/bin/sh

_mode=
_crontab_dir=
_user=
_address=

while getopts m:c:u:a: opts;do
	case $opts in
		m)
			_mode=${OPTARG}
			;;
		c)
			_crontab_dir=${OPTARG}
			;;
		u)
			_user=${OPTARG}
			;;
		a)
			_address=${OPTARG}
			;;
		*)
			echo invalid option
			exit 1
			;;
	esac
done

if [ -z ${_crontab_dir} ];then
#    echo "crontab dir not specified, use /etc/crontabs"
    _crontab_dir=/etc/crontabs
fi

if [ -z ${_user} ];then
    _user=`whoami`
    echo "Current user: ${_user}"
fi

gen_st_get_mate_ip() {
	local path=/bin/st_get_mate_ip
	cat > $path << EOF
#!/bin/sh

main_ip="192.168.1.1 192.168.2.1"
backup_ip="192.168.1.100 192.168.2.100"

mode=\$(st_env_get mode)
ip_candidates=
if [ "\${mode}" == "main" ];then
	ip_candidates="\${backup_ip}"
elif [ "\${mode}" == "backup" ];then
	ip_candidates="\${main_ip}"
else
	exit 1
fi

for i in \${ip_candidates};do
	ping -c 2 -w 2 \$i > /dev/null
	if [ \$? -eq 0 ];then
		echo \$i
		exit 0
	fi
done

exit 1

EOF
	chmod +x $path
}




gen_st_get_and_test_main_sc_load_routine() {
    local path=/bin/st_get_and_test_main_sc_load_routine
    cat > $path << EOF
#!/bin/sh

mode=\$(st_env_get mode)
if [ "\${mode}" == "main" ];then
    exit 1
fi

mate_ip=\$(st_get_mate_ip)
if [ \$? -ne 0 ];then
	exit 1
fi

self_cur_boot_load=\$(st_env_get cur_boot_load)

if [ -z "\${self_cur_boot_load}" ];then
    exit 1
fi


main_sc_boot_load=
self_cur_boot_flag=\$(st_env_get cur_boot_flag)
self_cur_boot_dir=/sino/appload/\${self_cur_boot_flag}

replace_diff_load() {
	local this_load=\$1
    if [ -f "\${this_load}" ];then
		#check md5sum first
		if ! st_check_file \${this_load};then
			return 1
		fi

        if st_test_diff_path \${self_cur_boot_load} \${this_load};then
			logger -t \$0 updating \$(md5sum \${self_cur_boot_load})
            rm -rf \${self_cur_boot_dir}
            mkdir -p \${self_cur_boot_dir}
			cp \${this_load} \${self_cur_boot_dir} && logger -t \$0 success update \$(md5sum \${this_load})
            sync
			return 0
        else
			return 1
        fi
    else
		return 1
    fi
}


if [ "\$(st_env_get mode)" == "backup" ];then
	#passive check /sino/main_stuff
    main_sc_boot_load=\`find  /sino/main_stuff -name *.gz | xargs realpath\`
	replace_diff_load \$main_sc_boot_load
	if [ \$? -eq 0 ];then
		exit 0
    else
        rm -rf /sino/main_stuff/*
	fi

	#active get
	init_get=\$(st_env_get backup_init_get)
	if [ "\${init_get}" == "0" ];then
		exit 1
	fi
#	times=\$(st_env_get backup_get_delay)
#	if [ "\$times" -ne 5 ];then
#		times=\`expr \$times + 1\`
#		st_env_put backup_get_delay \$times
#		exit 1
#	else
#		st_env_put backup_get_delay 0
#	fi
    rm -rf /sino/backup_stuff
    st_get -u root -m ncftp -r /sino/backup_stuff -l /sino -R -h \${mate_ip}
    if [ \$? -eq 0 ];then
		st_env_put backup_init_get 0
        main_sc_boot_load=\`find  /sino/backup_stuff -name *.gz | xargs realpath\`
		replace_diff_load \$main_sc_boot_load
		exit \$?
	else
		exit 1
    fi
else
    exit 1
fi

exit 1

EOF
    chmod +x ${path}
}

gen_st_put_fresh_load_to_backup_routine() {
	local path=/bin/st_put_fresh_load_to_backup_routine
	cat > $path << EOF
#!/bin/sh
mode=\$(st_env_get mode)
if [ "\$mode" == "backup" ];then
	exit 0
fi

mate_ip=\$(st_get_mate_ip)
if [ \$? -ne 0 ];then
	exit 1
fi

load=\$(st_env_get fresh_load_path)
check_file=\${load}.st_md5sum

if [ -z "\$load" ];then
	exit 1
elif [ ! -f \$load ];then
	exit 1
else
	st_gen_check_file \$load
	st_put -u root -m ncftp -h \${mate_ip} -l \$load         -r /sino/main_stuff/\$(basename \$load) && logger -t \$0 success put \$(md5sum \$load)
	st_put -u root -m ncftp -h \${mate_ip} -l \${check_file} -r /sino/main_stuff/\$(basename \$load).st_md5sum && logger -t \$0 success put \${check_file}
	rm \${check_file} -f
	exit \$?
fi

EOF
	chmod +x $path
}




#gen st_get in /bin
gen_st_get() {
	local path=/bin/st_get
	cat > $path << EOF
#!/bin/sh
_method=
_local_path=
_remote_path=
_host=
_isR=
_user=
_passwd=
_isSync=

_sync_lock=/tmp/st_get.lock

exit_st_get() {
    local ret=\$1
    if [ ! -z "\${_isSync}" ];then
        rm -f \${_sync_lock}
    fi

    exit \$ret
}

test_is_sync() {
    if [ -f \${_sync_lock} ];then
        return 1
    else
        return 0
    fi
}



#test_is_sync() {
#    local ret=\`ps aux | grep ".*st_get.* -s.*" | sed '/grep.*st_get.*-s.*/d'\`
#    ret=\`echo "\$ret" | wc -l\`
#    if [ \$ret -eq 2 ];then
#        return 0
#    else
#        return 1
#    fi
#}


while getopts m:l:r:h:Rsu:p: opts;do
	case \$opts in
		m)
			_method=\${OPTARG}
			;;
		l)
			_local_path=\${OPTARG}
			;;
		r)
			_remote_path=\${OPTARG}
			;;
		h)
			_host=\${OPTARG}
			;;
		R)
			_isR=1
			;;
        s)
            #sync mode
            if test_is_sync;then
                touch \${_sync_lock}
                _isSync=1
            else
                echo only one sync instance is allowed
                exit_st_get 1
            fi
            ;;
        u)
            _user=\${OPTARG}
            ;;
        p)
            _passwd=\${OPTARG}
            ;;
		*)
			echo invalid option
			exit_st_get 1
			;;
	esac
done

if [ -z "\${_local_path}" ];then
    echo "local path not specified"
    exit_st_get 1
fi
if [ -z "\${_remote_path}" ];then
	_remote_path=\${_local_path}
fi
if [ -z "\${_host}" ];then
    echo "host not specified"
    exit_st_get 1
fi


u_p_args=
get_user_and_passwd_args() {
    if [ -z "\${_user}" ];then
        u_p_args=
    else
        u_p_args="-u \${_user}"
        if [ -z "\${_passwd}" ];then
            u_p_args="\${u_p_args} -p ''"
        else
            u_p_args="\${u_p_args} -p \${_passwd}"
        fi
    fi
}
get_user_and_passwd_args

get_ncftp() {
    local local_path=\$1
    local remote_path=\$2
    local host=\$3
	local isR=\$4
    local args="\${u_p_args} "
    #10 secs timeout
    args="\${args} -t 10"
    #get a regular file
	if [ -z "\${isR}" ];then
        rm -rf \${local_path}
		args="-C \${args} \${host} \${remote_path} \${local_path}"
	else
        #local path can not be a file in Recursive mode
        if [ -f \${local_path} ];then
            return 1
        fi
		args="-R \${args} \${host} \${local_path} \${remote_path}"
	fi
    #quietly transform...
    ncftpget \${args} 2>/dev/null

    return \$?
}


case \${_method} in
    ncftp)
#        echo "Getting \${_host}:\${_remote_path} to \${_local_path}"
        get_ncftp \${_local_path} \${_remote_path} \${_host} \${_isR}
		ret=\$?
        exit_st_get \$ret
        ;;
    *)
        echo "Unknown method: \${_method}"
        exit_st_get 1
        ;;
esac

EOF
	chmod +x ${path}
}

#gen st_put in /bin
gen_st_put() {
	local path=/bin/st_put
	cat > ${path} << EOF
#!/bin/sh
_method=
_local_path=
_remote_path=
_host=
_isR=
_user=
_passwd=
_isSync=

_sync_lock=/tmp/st_put.lock

exit_st_put() {
    local ret=\$1
    if [ ! -z "\${_isSync}" ];then
        rm -f \${_sync_lock}
    fi

    exit \$ret
}

test_is_sync() {
    if [ -f \${_sync_lock} ];then
        return 1
    else
        return 0
    fi
}


while getopts m:l:r:h:sRu:p: opts;do
	case \$opts in
		m)
			_method=\${OPTARG}
			;;
		l)
			_local_path=\${OPTARG}
			;;
		r)
			_remote_path=\${OPTARG}
			;;
		h)
			_host=\${OPTARG}
			;;
        s)
            #sync mode
            if test_is_sync;then
                touch \${_sync_lock}
            else
                echo only one sync instance is allowed
                exit_st_put 1
            fi
            ;;
        R)
            _isR=1
            ;;
        u)
            _user=\${OPTARG}
            ;;
        p)
            _passwd=\${OPTARG}
            ;;
		*)
			echo invalid option
			exit_st_put 1
			;;
	esac
done

if [ -z "\${_local_path}" ];then
    echo "local path not specified"
    exit_st_put 1
fi

if [ -z "\${_remote_path}" ];then
    echo "remote path not specified"
    exit_st_put 1
fi

if [ -z "\${_host}" ];then
    echo "host not specified"
    exit_st_put 1
fi

u_p_args=
get_user_and_passwd_args() {
    if [ -z "\${_user}" ];then
        u_p_args=
    else
        u_p_args="-u \${_user}"
        if [ -z "\${_passwd}" ];then
            u_p_args="\${u_p_args} -p ''"
        else
            u_p_args="\${u_p_args} -p \${_passwd}"
        fi
    fi
}
get_user_and_passwd_args

put_ncftp() {
    local local_path=\$1
    local remote_path=\$2
    local host=\$3
    local isR=\$4
    #user root, no passwd, timeout 20 seconds
    local args="\${u_p_args} "
    #10 secs timeout
    args="\${args} -t 10"
	if [ ! -z "\${isR}" ];then
        if [ -f \${local_path} ];then
            exit_st_put 1
        fi
		args="-R \${args} \${host} \${remote_path} \${local_path}"
	else
		args="-C \${args} \${host} \${local_path} \${remote_path}"
	fi
    #quietly transform...
    ncftpput \${args} 2>/dev/null
    return \$?
}


case \${_method} in
    ncftp)
        echo "Putting \${_local_path} to \${_remote_path} in \${_host}"
        put_ncftp \${_local_path} \${_remote_path} \${_host} \${_isR}
		ret=\$?
        return \$ret
        ;;
    *)
        echo "Unknown method: \${_method}"
        exit_st_put 1
        ;;
esac
EOF
	chmod +x ${path}
}



gen_cron() {
	local path=/bin/cron.sh
	cat > ${path} << EOF
#!/bin/sh

get_pid_column() {
	local ps_banner=\`ps aux | sed '1p' -n\`
	local pid_col=1
	local i
	for i in \$ps_banner;do
		if [ "\$i" == "PID" ];then
			break;
		fi
		pid_col=\`expr \$pid_col + 1\`
	done

	echo \$pid_col
}

get_crond_pid() {
	local pid
	local pid_col=\$(get_pid_column)
	pid=\`ps aux | grep "crond" | sed '/grep/d' | awk -v col=\$pid_col '{print \$col}'\`
	echo \$pid
}


start() {
    crond
}

stop() {
	local pid=\$(get_crond_pid)
	for i in \$pid;do
		kill \$i
	done
}

restart() {
    stop
    start
}

case \$1 in
	start)
		start;;
	stop)
		stop;;
	restart)
		restart;;
	*)
		echo Unknown opt
		exit 1
		;;
esac

exit 0

EOF

	local crontab_dir=$(crond --help 2>&1 | grep -o ".*crontabs.*" | grep -o "/.*")
    mkdir -p ${crontab_dir}
    cat > ${crontab_dir}/${USER} << EOF
#*/1 * * * * test "\$(st_env_get mode)" == "backup" && { st_env_put conf_sync_flag 1    ; st_get_and_test -u root -l /sino/backup/config/main.conf  ; st_env_put conf_sync_flag 0; }
#*/1 * * * * test "\$(st_env_get mode)" == "backup" && { st_env_put mibdata_sync_flag 1 ; st_get_and_test -u root -l /var/run/product/mibdata -R    ; st_env_put mibdata_sync_flag 0; }
*/1 * * * *  st_update_resource
*/1 * * * *  st_check_telnetd
EOF


	chmod +x ${path}
}


gen_ftp_server() {
	local path=/bin/ftpd.stsrv
	cat > $path << EOF
#!/etc/st_scripts/stsrv

. /lib/st_func/common

mode=
map_user=
address=
port=21
root_dir=/
while getopts a:p:d:n: opts;do
	case \$opts in
		a)
			address="\$OPTARG"
			;;
		p)
			port=\${OPTARG}
			;;
		d)
			root_dir=\${OPTARG}
			;;
		n)
			#busybox version greater than 1.27 support anonymous mode
			st_func_common_test_version \$(st_func_common_get_busybox_version) 1.27
			if ! test \$? -gt 1;then
				mode=anonymous
				map_user=\${OPTARG}
			fi
			;;
		*)
			echo invalid opt
			exit 1
			;;
	esac
done

if test -z "\$address";then
	if test "\$stsrv_action" == "start";then
		logger -t ftpd.stsrv -s "Address is null"
		exit 1
	fi
fi

get_busybox_ftpd_pid() {
	local ftpd_pid
	local pid_col=\$(st_func_common_get_pid_column)
	ftpd_pid=\`ps aux | grep "tcpsvd .* ftpd.*" | sed '/grep/d' | awk -v col=\$pid_col '{print \$col}'\`
	echo \$ftpd_pid
}

start() {
	local i
	for i in \${address};do
		setsid tcpsvd \$i \${port} ftpd \${mode:+-a \${map_user}} -w -S \${root_dir} &
	done
}

stop() {
	local ftpd_pid="\$(get_busybox_ftpd_pid)"
	for i in \${ftpd_pid};do
		kill \$i
	done
}

restart() {
	stop
	start
}

EOF
	chmod +x $path
}

gen_ntpd_service()
{
    local path=/bin/ntpd.stsrv
    cat > $path << EOF
#!/etc/st_scripts/stsrv

interface=
peer=

while getopts i:p: opts;do
    case \$opts in
        i)
            interface="\${OPTARG}"
            ;;
        p)
            peer="\${OPTARG}"
            ;;
        *)
            logger -t ntpd.stsrv "Unknown option"
            exit 1
            ;;
    esac
done

start()
{
    if test -n "\$interface";then
        for i in \${interface};do
            cmd="ntpd -I \$i"
            logger -t ntpd.stsrv "\$cmd"
            eval \$cmd
        done
    fi
    if test -n "\$peer";then
        for i in \$peer;do
            cmd="ntpd -p \$i"
            logger -t ntpd.stsrv "\$cmd"
            eval \$cmd
        done
    fi
}

stop()
{
    killall ntpd
}

restart()
{
    stop
    start
}
EOF
    chmod +x $path
}


gen_st_get_and_test() {
    local path=/bin/st_get_and_test
	cat > $path << EOF
#!/bin/sh

#st_get_and_test [-R] [-u user] [-p passwd] -l local_path [-r remote_path] [-h remote_host]

local_path=
remote_path=
remote_host=
isR=
user=
password=


#log file action
#log_file=/tmp/st_get_and_test.log
#touch \$log_file
#log_size=\`ls -sail \$log_file | awk '{print \$7}'\`
#if [ \$log_size -gt 30000 ];then
#    rm -f \$log_file
#    touch \$log_file
#fi

while getopts Ru:p:l:r:h: opts;do
	case \$opts in
		R)
			isR=1
			;;
		u)
			user=\${OPTARG}
			;;
		p)
			password=\${OPTARG}
			;;
		l)
			local_path=\${OPTARG}
			;;
		r)
			remote_path=\${OPTARG}
			;;
		h)
			remote_host=\${OPTARG}
			;;
		*)
			echo invalid option
            exit 1
			;;
	esac
done

#num_shift=\`expr \$OPTIND - 1\`
#shift \${num_shift}

#this is derived from gen_backstuff.sh
if [ -z "\${remote_host}" ];then
	remote_host=\$(st_get_mate_ip)
	if [ \$? -ne 0 ];then
		exit 1
	fi
fi

#local path must be specified
if [ -z "\$local_path" ];then
#	echo [\`date\`] local_path not specified | tee -a \$log_file
	logger -t \$0 local_path not specified
	exit 1
fi

#remote path is the same as local path by default
if [ -z "\$remote_path" ];then
	remote_path=\${local_path}
fi

#avoid duplicate tmp work dir
local_tmp_dir=
get_loacl_tmp_dir() {
	ran_num=\`tr -cd 0-9 < /dev/urandom | head -c 4\`
	local_tmp_dir=/tmp/get_and_test.\${ran_num}
	if ! mkdir \${local_tmp_dir};then
		sleep 1
		get_loacl_tmp_dir
	fi
}
get_loacl_tmp_dir

local_tmp_path=\${local_tmp_dir}/\`basename \$remote_path\`

u_p_args=
get_user_and_passwd_args() {
    if [ -z "\${user}" ];then
        u_p_args=
    else
        u_p_args="-u \${user}"
        if [ -z "\${password}" ];then
            u_p_args="\${u_p_args} -p ''"
        else
            u_p_args="\${u_p_args} -p \${password}"
        fi
    fi
}

get_user_and_passwd_args

exit_st_get_and_test() {
    local ret=\$1

	rm -rf \${local_tmp_dir}
    exit \$ret
}


#sync get
if [ -z "\${isR}" ];then
	st_get \${u_p_args}  -m ncftp -l \$local_tmp_path  -r \$remote_path -h \${remote_host} > /dev/null
else
	st_get \${u_p_args}  -m ncftp -l \$local_tmp_dir  -r \$remote_path -h \${remote_host} -R > /dev/null
fi

ret=\$?
if [ \$ret -ne 0 ];then
	#echo [\`date\`] st_get failed while updating remote \${remote_host}:\${remote_path} to local \${local_path} | tee -a \$log_file
	logger -t \$0 st_get failed while updating remote \${remote_host}:\${remote_path} to local \${local_path}
    exit_st_get_and_test 1
fi

if [ \$ret -eq 0 ];then
    if st_test_diff_path -m diff \$local_tmp_path \$local_path;then
		rm -rf \${local_path}
		if [ -z "\${isR}" ];then
			cp \${local_tmp_path} \${local_path}
		else
			cp \${local_tmp_path} \${local_path} -r
		fi
		#echo [\`date\`] success update remote \${remote_host}:\${remote_path} to local \${local_path} | tee -a \$log_file
		logger -t \$0 success update remote \${remote_host}:\${remote_path} to local \${local_path}, size \$(ls -sailh \${local_path} | awk '{print \$7}')
		exit_st_get_and_test 0
    else
        #echo [\`date\`] same remote \${remote_host}:\${remote_path} and local \${local_path} | tee -a \$log_file
		logger -t \$0 same remote \${remote_host}:\${remote_path} and local \${local_path}, size \$(ls -sailh \${local_path} | awk '{print \$7}')
		exit_st_get_and_test 0
    fi
else
    #echo [\`date\`] st_get remote \${remote_host}:\${remote_path} fail | tee -a \$log_file
	logger -t \$0 st_get remote \${remote_host}:\${remote_path} fail
	exit_st_get_and_test 1
fi

EOF

    chmod +x $path
}


gen_scripts() {
	gen_st_get_mate_ip
	gen_st_get
	gen_st_put
	gen_ftp_server
	gen_cron
	gen_st_get_and_test
    gen_ntpd_service
}

gen_scripts

