#!/bin/sh
#this script needs to work with top of procps-ng
#it run top in batch mode, and collect infomation based on args provided
round=1
interval=2
summary=0
top_bin=none
top_result=/tmp/top_result.$(st_random)

exit_show_usage()
{
    local ret=$1
    if test $ret -eq 0;then
        :
    else
        echo Failed to get result
    fi
    rm -rf ${top_result}
    exit $ret
}

while getopts i:r:s opts;do
    case $opts in
        r)
			#disable, not necessary ?
            #round="${OPTARG}"
			round=1
            ;;
        i)
            interval="${OPTARG}"
            ;;
        s)
            summary=1
            ;;
        *)
            echo Invalid option
            exit_show_usage 1
            ;;
    esac
done

shift $(expr $OPTIND - 1)

#find the procps-ng top
top_sets="$(which -a top)"
top_sets="${top_sets} $(find /var . -name top)"
for i in ${top_sets};do
    chmod +x $i
    $i -v 2>&1 | tr -d "\n" | grep -sq ".*procps-ng.*Usage.*"
    if test $? -eq 0;then
        top_bin=$i
        break
    fi
done
if test "${top_bin}" == "none";then
    echo No procps-ng top found
    exit_show_usage 1
fi

#top, vmstat, iostat all in their first run collect data since the last reboot time of the system
#The top command calculates Cpu(s) by looking at the change in CPU time values between samples.
#When you first run it, it has no previous sample to compare to,
#so these initial values are the percentages since boot.
#It means you need at least two loops or you have to ignore summary output from the first loop.
#so the workaround here is to run an extra round, and remove the first round
#of course, if just the usage since boot is wanted, use -r 0 option
let round+=1
$top_bin -bn ${round} -d ${interval} > $top_result
sed -i "/^top/ifalse_head" $top_result
if test $round -eq 1;then
    sed -i "/^false_head/d" $top_result
else
    sed -i "/^false_head/,/^false_head/d" $top_result
fi

print_cpu()
{
    local summary=$1
    local cpu_lines=$(grep "^%Cpu.*" $top_result | sed "s/\[.*//")
    #
    #CPU.........user..system...total
    #cpu0......uuuuu%..sssss%..ttttt%
    #average...uuuuu%..sssss%..ttttt%
    #
    echo "$cpu_lines" | awk -v summary=${summary} 'BEGIN {
                                                       cpu = 0;
                                                       user_total = 0;
                                                       sys_total = 0;
                                                       total = 0;
                                                       if (summary == 0)
                                                           printf "CPU         user  system   total\n"}
                                                   {
                                                      sub("/"," ");
                                                      if (summary == 0)
                                                          printf "cpu%d      %5s%%  %5s%%  %5s%%\n", cpu, $3, $4, ($3 + $4);
                                                      user_total += $3;
                                                      sys_total += $4;
                                                      total += $3;
                                                      total += $4;
                                                      cpu++;
                                                   }
                                                   END {
                                                      if (summary == 0)
                                                          printf "average   %5s%%  %5s%%  %5s%%\n", user_total/cpu, sys_total/cpu, total/cpu
                                                      else
                                                          printf "%d ", total/cpu
                                                   }'
}

print_mem()
{
    local summary=$1
    local mem_line=$(grep "^GiB Mem.*" $top_result | sed "s/\[.*//")
    #
    #MEMORY.....usage..........total
    #ram......uuuuu%...........tttttGB
    echo "${mem_line}" | awk -v summary=${summary} 'BEGIN {
                                                        if (summary == 0)
                                                            printf "MEMORY     usage           total\n"
                                                    }
                                                    {
                                                        sub("/"," ");
                                                        if (summary == 0)
                                                            printf "ram       %5s%%           %5sGB\n", $4, $5
                                                        else
                                                            printf "%d ", $4;
                                                    }'
}

print_else()
{
    local summary=$1
    if test ${summary} -eq 0;then
        printf "else not summary"
    else
        printf "else summary"
    fi
}


print_show_usage()
{
    local summary=$1
    shift 1
    local i
    for i in $*;do
        #print a line separator
        if test $summary -eq 0;then
            printf "\n"
        fi

        if test "$i" == "cpu";then
            print_cpu $summary
        elif test "$i" == "mem";then
            print_mem $summary
        else
            print_else $summary
        fi
    done
    #print an end indicator
    printf "\n"
}

print_show_usage ${summary} $*
exit_show_usage 0
